class HTTPException(Exception):
    pass


class SentencesException(Exception):
    pass


class QAException(Exception):
    pass


class AnalysisException(Exception):
    pass


class AudioFileException(Exception):
    pass


class VoiceRecognitionException(Exception):
    pass


class VoicePronunciationException(Exception):
    pass
